@@ -1,4 +1,4 @@
-import contextlib, datetime, itertools, methodtools, os, pathlib, random, re, subprocess, sys, time, uuid
+import contextlib, datetime, itertools, os, pathlib, random, re, subprocess, sys, time, uuid
 if sys.platform != "cygwin":
   import psutil
 
@@ -13,6 +13,12 @@ def rm_missing_ok(path):
 
 def SLURM_JOBID():
   return os.environ.get("SLURM_JOBID", None)
+def CONDOR_JOBINFO():
+  if "_CONDOR_SCRATCH_DIR" not in os.environ: return None
+  try:
+    return os.environ["CONDOR_CLUSTERID"], os.environ["CONDOR_PROCID"]
+  except KeyError:
+    raise OSError("Please put 'environment = CONDOR_cLUSTERID=$(ClusterId) CONDOR_PROCID=$(ProcId)' in your condor submission script")
 
 def cpuid():
   node = uuid.getnode()
@@ -32,30 +38,46 @@ def cpuid():
   raise ValueError("Couldn't find a cpuid using any of the methods we know about")
 
 def jobinfo():
-  if SLURM_JOBID() is not None:
-    return "SLURM", 0, SLURM_JOBID()
+  slurm = SLURM_JOBID()
+  if slurm is not None:
+    return "SLURM", 0, slurm
+  condor = CONDOR_JOBINFO()
+  if condor is not None:
+    return "CONDOR", condor[0], condor[1]
   return sys.platform, cpuid(), os.getpid()
 
 class JobLock(object):
   defaultcorruptfiletimeout = None
 
-  def __init__(self, filename, *, outputfiles=[], checkoutputfiles=True, inputfiles=[], checkinputfiles=True, corruptfiletimeout=None, mkdir=False):
+  def __init__(self, filename, *, outputfiles=[], checkoutputfiles=True, inputfiles=[], checkinputfiles=True, prevsteplockfiles=[], corruptfiletimeout=None, mkdir=False, dosqueue=True, cachesqueue=False):
     self.filename = pathlib.Path(filename)
     self.outputfiles = [pathlib.Path(_) for _ in outputfiles]
     self.inputfiles = [pathlib.Path(_) for _ in inputfiles]
+    self.prevsteplockfiles = [pathlib.Path(_) for _ in prevsteplockfiles]
     self.checkoutputfiles = outputfiles and checkoutputfiles
     self.checkinputfiles = inputfiles and checkinputfiles
+    self.checkprevsteplockfiles = prevsteplockfiles
     self.removed_failed_job = False
     if corruptfiletimeout is None:
       corruptfiletimeout = self.defaultcorruptfiletimeout
     self.corruptfiletimeout = corruptfiletimeout
     self.mkdir = mkdir
+    self.dosqueue = dosqueue
+    self.cachesqueue = cachesqueue
+    self.sublockkwargs = {
+      "checkoutputfiles": checkoutputfiles,
+      "checkinputfiles": checkinputfiles,
+      "mkdir": mkdir,
+      "dosqueue": dosqueue,
+      "cachesqueue": cachesqueue,
+      "corruptfiletimeout": corruptfiletimeout,
+    }
     self.__reset()
 
   def __reset(self):
     self.fd = self.f = None
     self.bool = False
-    self.__inputsexist = self.__outputsexist = self.__oldjobinfo = None
+    self.__inputsexist = self.__outputsexist = self.__prevsteplockfilesexist = self.__oldjobinfo = None
 
   @property
   def wouldbevalid(self):
@@ -82,17 +104,18 @@ class JobLock(object):
       if exceptions: raise
       return None, None, None
 
-  @methodtools.lru_cache()
   @property
   def outputsexist(self):
     return self.__outputsexist
 
-  @methodtools.lru_cache()
   @property
   def inputsexist(self):
     return self.__inputsexist
 
-  @methodtools.lru_cache()
+  @property
+  def prevsteplockfilesexist(self):
+    return self.__prevsteplockfilesexist
+
   @property
   def oldjobinfo(self):
     return self.__oldjobinfo
@@ -132,10 +155,10 @@ class JobLock(object):
     if not filenames: return
     filenames.sort(key=n_from_filename, reverse=True)
 
-    with JobLock(self.filename.with_suffix(".cleanup.lock")) as lock:
+    with JobLock(self.filename.with_suffix(".cleanup.lock"), **self.sublockkwargs) as lock:
       if not lock: return
       for filename in filenames:
-        with JobLock(filename, corruptfiletimeout=self.corruptfiletimeout) as lock:
+        with JobLock(filename, **self.sublockkwargs) as lock:
           if not lock:
             break
 
@@ -150,6 +173,10 @@ class JobLock(object):
       self.__inputsexist = {_: _.exists() for _ in self.inputfiles}
       if not all(self.inputsexist.values()):
         return self
+    if self.checkprevsteplockfiles:
+      self.__prevsteplockfilesexist = {_: not JobLock(_, **self.sublockkwargs).wouldbevalid for _ in self.prevsteplockfiles}
+      if any(self.prevsteplockfilesexist.values()):
+        return self
     if self.mkdir:
       self.filename.parent.mkdir(parents=True, exist_ok=True)
     try:
@@ -160,7 +187,7 @@ class JobLock(object):
       #a race condition: two jobs could be looking if the previous
       #job failed at the same time, and one of them could remove
       #the lock created by the other one
-      with JobLock(self.iterative_lock_filename, corruptfiletimeout=self.corruptfiletimeout) as iterative_lock:
+      with JobLock(self.iterative_lock_filename, **self.sublockkwargs) as iterative_lock:
         if not iterative_lock: return self
         try:
           self.__oldjobinfo = self.runningjobinfo(exceptions=True)
@@ -189,7 +216,7 @@ class JobLock(object):
           else:
             return self
         else:
-          if jobfinished(*self.oldjobinfo):
+          if jobfinished(*self.oldjobinfo, dosqueue=self.dosqueue, cachesqueue=self.cachesqueue):
             for outputfile in self.outputfiles:
               rm_missing_ok(outputfile)
             rm_missing_ok(self.filename)
@@ -236,12 +263,20 @@ class JobLock(object):
     return {
       "outputsexist": self.outputsexist,
       "inputsexist": self.inputsexist,
+      "prevsteplockfilesexist": self.prevsteplockfilesexist,
       "oldjobinfo": self.oldjobinfo,
       "removed_failed_job": self.removed_failed_job,
     }
 
-def jobfinished(jobtype, cpuid, jobid):
+__knownrunningslurmjobs = set()
+
+def clear_slurm_running_jobs_cache():
+  __knownrunningslurmjobs.clear()
+
+def jobfinished(jobtype, cpuid, jobid, *, dosqueue=True, cachesqueue=True):
   if jobtype == "SLURM":
+    if cachesqueue and jobid in __knownrunningslurmjobs: return False #assume job is still running
+    if not dosqueue: return None #don't know if the job finished
     try:
       output = subprocess.check_output(["squeue", "--job", str(jobid), "--format", "jobid,state", "--noheader"], stderr=subprocess.STDOUT)
       for line in output.split(b"\n"):
@@ -258,6 +293,7 @@ def jobfinished(jobtype, cpuid, jobid):
             #this can happen if the job was cancelled due to node failure and was automatically resubmitted
             return True #job is not currently running
           else:
+            __knownrunningslurmjobs.add(jobid)
             return False #job is still running
       return True #job is finished
     except FileNotFoundError:  #no squeue
@@ -267,6 +303,28 @@ def jobfinished(jobtype, cpuid, jobid):
         return True #job is finished
       print(e.output)
       raise
+  elif jobtype == "CONDOR":
+    if not dosqueue: return None #don't know if the job finished
+    try:
+      output = subprocess.check_output(["condor_q", "-nobatch", "-run"], stderr=subprocess.STDOUT)
+      for line in output.split(b"\n"):
+        line = line.strip()
+        if not line: continue
+        if line.startswith(b"-- Schedd:"): continue
+        if line.startswith(b"ID "): continue
+        runningjobid = line.split()[0]
+        try:
+          runningclusterid, runningprocid = (int(_) for _ in runningjobid.split(b"."))
+        except ValueError:
+          return None #don't know if the job finished, probably a temporary glitch in squeue
+        if runningclusterid == cpuid and runningprocid == jobid:
+          return False #job is still running
+      return True #job is finished
+    except FileNotFoundError:  #no squeue
+      return None  #we don't know if the job finished
+    except subprocess.CalledProcessError as e:
+      print(e.output)
+      raise
   else:
     myjobtype, mycpuid, myjobid = jobinfo()
     if myjobtype != jobtype: return None #we don't know if the job finished
@@ -289,7 +347,7 @@ def jobfinished(jobtype, cpuid, jobid):
 class JobLockAndWait(JobLock):
   defaultsilent = False
 
-  def __init__(self, name, delay, *, printmessage=None, task="doing this", maxiterations=1000, silent=None, **kwargs):
+  def __init__(self, name, delay, *, printmessage=None, task="doing this", maxiterations=1000, silent=None, waitforinputs=False, **kwargs):
     super().__init__(name, **kwargs)
     self.delay = delay
     if printmessage is None:
@@ -299,6 +357,7 @@ class JobLockAndWait(JobLock):
     if silent is None:
       silent = self.defaultsilent
     self.__silent = silent
+    self.__waitforinputs = waitforinputs
     self.niterations = 0
     self.maxiterations = maxiterations
 
@@ -309,7 +368,18 @@ class JobLockAndWait(JobLock):
       result = super().__enter__()
       if result:
         return result
-      if not self.__silent: print(self.__printmessage)
+      elif self.checkoutputfiles and self.outputsexist is not None and all(self.outputsexist.values()):
+        return result
+      elif self.checkinputfiles and self.inputsexist is not None:
+        missinginputs = [k for k, v in self.inputsexist.items() if not v]
+        if missinginputs:
+          message = f"Some input files are missing: {', '.join(str(_) for _ in missinginputs)}."
+          if self.__waitforinputs:
+            if not self.__silent: print(f"{message} Waiting {self.delay} seconds.")
+          else:
+            raise FileNotFoundError(message)
+      else:
+        if not self.__silent: print(self.__printmessage)
       time.sleep(self.delay)
 
 def clean_up_old_job_locks(folder, glob="*.lock_*", howold=datetime.timedelta(days=7), dryrun=False, silent=False):
