@@ -1,5 +1,5 @@
 import contextlib, datetime, multiprocessing, os, pathlib, subprocess, tempfile, time, unittest
-from job_lock import clean_up_old_job_locks, JobLock, JobLockAndWait, jobinfo, MultiJobLock, slurm_clean_up_temp_dir, slurm_rsync_input, slurm_rsync_output
+from job_lock import clean_up_old_job_locks, clear_slurm_running_jobs_cache, JobLock, JobLockAndWait, jobinfo, MultiJobLock, slurm_clean_up_temp_dir, slurm_rsync_input, slurm_rsync_output
 
 class TestJobLock(unittest.TestCase, contextlib.ExitStack):
   def __init__(self, *args, **kwargs):
@@ -14,6 +14,7 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     self.slurm_tmpdir = self.tmpdir/"slurm_tmpdir"
     self.slurm_tmpdir.mkdir()
     os.environ["TMPDIR"] = os.fspath(self.slurm_tmpdir)
+    clear_slurm_running_jobs_cache()
   def tearDown(self):
     del self.tmpdir
     self.close()
@@ -120,9 +121,13 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
 
     with open(fn1, "w") as f:
       f.write("SLURM 0 1234568")
+    with open(fn2, "w") as f:
+      f.write("SLURM 0 1234568")
     with JobLock(fn1, outputfiles=[output1, output2], dosqueue=False) as lock:
       self.assertFalse(lock)
-      self.assertEqual(lock.debuginfo, {"outputsexist": None, "inputsexist": None, "prevsteplockfilesexist": None, "oldjobinfo": ("SLURM", 0, 1234568), "removed_failed_job": False})
+      self.assertEqual(lock.debuginfo, {"outputsexist": None, "inputsexist": None, "prevsteplockfilesexist": None, "oldjobinfo": None, "removed_failed_job": False})
+    self.assertTrue(fn1.exists())
+    self.assertTrue(fn2.exists())
     with JobLock(fn1, outputfiles=[output1, output2]) as lock:
       self.assertTrue(lock)
       self.assertEqual(lock.debuginfo, {"outputsexist": None, "inputsexist": None, "prevsteplockfilesexist": None, "oldjobinfo": ("SLURM", 0, 1234568), "removed_failed_job": True})
@@ -233,6 +238,68 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     with JobLock(self.tmpdir/"lock6.lock") as lock6:
       self.assertTrue(lock6)
 
+  def testcondor(self):
+    dummycondor_q = """
+      #!/bin/bash
+      echo '
+        -- Schedd: my schedd
+         ID      OWNER            SUBMITTED     RUN_TIME ST PRI SIZE CMD
+         1234567.1
+         1234568.1
+      '
+    """.lstrip()
+    with open(self.tmpdir/"condor_q", "w") as f:
+      f.write(dummycondor_q)
+    (self.tmpdir/"condor_q").chmod(0o0777)
+    os.environ["PATH"] = f"{self.tmpdir}:"+os.environ["PATH"]
+
+    with open(self.tmpdir/"lock1.lock", "w") as f:
+      f.write("SLURM 1234567 1")
+    with open(self.tmpdir/"lock2.lock", "w") as f:
+      f.write("CONDOR 1234567 1")
+    with open(self.tmpdir/"lock3.lock", "w") as f:
+      f.write("CONDOR 12345678 1")
+    with open(self.tmpdir/"lock4.lock", "w") as f:
+      f.write("CONDOR 1234568 1")
+    with open(self.tmpdir/"lock5.lock", "w") as f:
+      f.write("CONDOR 1234567 2")
+
+    with JobLock(self.tmpdir/"lock1.lock") as lock1:
+      self.assertFalse(lock1)
+    with JobLock(self.tmpdir/"lock2.lock") as lock2:
+      self.assertFalse(lock2)
+    with JobLock(self.tmpdir/"lock3.lock") as lock3:
+      self.assertTrue(lock3)
+    with JobLock(self.tmpdir/"lock4.lock") as lock4:
+      self.assertFalse(lock4)
+    with JobLock(self.tmpdir/"lock5.lock") as lock5:
+      self.assertTrue(lock5)
+
+  def testCacheSqueue(self):
+    with open(self.tmpdir/"lock.lock", "w") as f:
+      f.write("SLURM 0 1234567")
+    with JobLock(self.tmpdir/"lock.lock") as lock:
+      self.assertFalse(lock)
+
+    dummysqueue = """
+      #!/bin/bash
+      echo '
+           1234567   RUNNING
+      '
+      sed -i s/1234567/1234568/g $0
+    """.lstrip()
+    with open(self.tmpdir/"squeue", "w") as f:
+      f.write(dummysqueue)
+    (self.tmpdir/"squeue").chmod(0o0777)
+    os.environ["PATH"] = f"{self.tmpdir}:"+os.environ["PATH"]
+
+    with JobLock(self.tmpdir/"lock.lock") as lock:
+      self.assertFalse(lock)
+    with JobLock(self.tmpdir/"lock.lock", cachesqueue=True) as lock:
+      self.assertFalse(lock)
+    with JobLock(self.tmpdir/"lock.lock") as lock:
+      self.assertTrue(lock)
+
   def testJobLockAndWait(self):
     with JobLockAndWait(self.tmpdir/"lock1.lock", 0.001, silent=True) as lock1:
       self.assertEqual(lock1.niterations, 1)
@@ -290,7 +357,8 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     touchlater(inputfile, iterationtime * 1.5)
     with JobLockAndWait(self.tmpdir/"lock3.lock", iterationtime, maxiterations=10, silent=True, inputfiles=[inputfile], waitforinputs=True) as lock3:
       self.assertTrue(lock3)
-      self.assertEqual(lock3.niterations, 3)
+      self.assertGreaterEqual(lock3.niterations, 3)
+      self.assertLessEqual(lock3.niterations, 4)
 
   def testCorruptFileTimeout(self):
     with open(self.tmpdir/"lock1.lock_2", "w"): pass
